/**
   This example illustrates the 2D elliptic problem on a rectangular
   domain and zero Dirichlet boundary conditions.
 */

#include <iostream>
#include <math.h>
#include <vector>
#include <limits>
#include <gmpxx.h>
#include "hdnum.hh"

#include "laplace.hh"

//! The domain function is passed to the model class. It defines the
//! computational domain Omega, especially needed, if Omega is a rectangular,
//! this is trivial
template<class N>
class LDomainFunction
{
public:

  LDomainFunction()
  {}

  typedef N number_type;
  bool evaluate(Vector<number_type> &) const
  {
    return true;
  }
};

//! The boundary functor determines for each grid boundary node,
//! whether it is a Dirichlet boundary node (otherwise it is assumed
//! to be a Neumann boundary node). It also provides the Dirichlet or
//! Neumann values.
template<class N>
class LBoundaryFunction
{
public:

  LBoundaryFunction()
  {}

  typedef N number_type;
  bool isDirichlet(const Vector<number_type> & ) const
  {
    return true;
  }

  number_type getDirichletValue(const number_type , 
                                const Vector<number_type> &) const
  {
    return 1;
  }

  Vector<number_type> getNeumannValue(const number_type , 
                                      const Vector<number_type> &) const
  {
    return 0; //is not used in this example
  }
};


using namespace hdnum; 
int main ()
{
  // The number type.
  typedef double Number; 

  // The spatial dimension.
  const int dim = 2;

  // The extent of the grid (meaning the grid domain spans the
  // quadrant [0,0] X [extent[0],extent[1]].
  Vector<Number> extent(dim);
  extent[0]=1.; extent[1]=1.;

  // The grid resolution. Here it is 31x31 nodes i.e. 30x30 cells.
  Vector<size_t> size(dim);
  size[0]=31; size[1]=31;

  std::cout << "Problem" << std::endl;
  {
    typedef LBoundaryFunction<Number> BoundaryFunction;
    typedef LDomainFunction<Number> DomainFunction;
    BoundaryFunction bf;
    DomainFunction df;
    typedef LaplaceCentralDifferences<Number,Number,DomainFunction,BoundaryFunction,dim>
      Model;

    std::cout << "Setting up the model" << std::endl;
    Model model(extent,size,df,bf);

    typedef StationarySolver<Model> Solver;
    std::cout << "Setting up the solver" << std::endl;
    Solver solver(model);

    std::cout << "Solving" << std::endl;
    solver.solve();
    Vector<Number> x = solver.get_state();

    std::cout << "Output" << std::endl;
    pde_gnuplot2d("laplace.gp",x,model.getGrid()); // output model result
  }

  return 0;
}
